// Include appropriate libraries
#include <iostream>
#include <vector>

#define _USE_MATH_DEFINES
#include <math.h>

#include <GL/glut.h>
#include <ctime>

// Include other classes required
#include "LightCycle.h"
#include "Trail.h"
#include "Disc.h"
#include "graphicsUtils.h"

// Grab access to the global list of pointers to objects
extern vector< LightCycle* > gCycles;
extern vector< Trail* > gTrails;
extern vector< Disc* > gDiscs;

using namespace std;

double convertAngleToRadians( float degrees )
{
	 return( (double) ( degrees * M_PI / 180.0 ) );
}

// light cycle CLASS - Constructor - initalize light cycles default values

LightCycle::LightCycle( float x = 0.0, float y = 0.0, float z = 0.0, float startangle = 0, int cycleNum = 0, float color1 = 0.0, float color2 = 0.0, float color3 = 0.0, float lightcolor1 = 0.0, float lightcolor2 = 0.0, float lightcolor3 = 0.0, float shading1 = 0.0, float shading2 = 0.0, float shading3 = 0.0 )
{
	speed = 0.03;
	bodysize = 0.4;
	wheelsize = bodysize / 4.0;
	tailsize = bodysize / 5.0;
	angle = startangle;
	
	sizeL = 0.5;
	sizeW = (bodysize / 6.0);

	name = "None";

	position[0] = x;
	position[1] = y;
	position[2] = z;

	defaultPosition[0] = 0.0;
	defaultPosition[1] = 0.0;
	defaultPosition[2] = 0.0;

	prevPosition[0] = 0.0;
	prevPosition[1] = 0.0;
	prevPosition[2] = 0.0;

	moveVertical = 0.0;
	moveHorizon = 0.0;

	color[0] = color1;
	color[1] = color2;
	color[2] = color3;
	
	lightcolor[0] = lightcolor1;
	lightcolor[1] = lightcolor2;
	lightcolor[2] = lightcolor3;
	
	shading[0] = shading1;
	shading[1] = shading2;
	shading[2] = shading3;
	
	disableColor[0] = 0.0;
	disableColor[1] = 0.0;
	disableColor[2] = 0.0;
	
	identify = cycleNum;
	
	lives = 1;

	if (cycleNum == 0)
	{
		leftKey = 'd';
		rightKey = 'a';
		discKey = ' ';
	}
	if (cycleNum == 1)
	{
		leftKey = '1';
		rightKey = '3';
		discKey = '0';
	}
	
	lastTimeFired = -10000;
	disabled = false;
	disableGame = false;
	disabledTime = -10000;
	shotDelay = 0.3;
	
	posOffsetX = 0.0;
	posOffsetY = 0.0;

	elapsedTime = 1.0;
}

// Draw Light Cycle
void LightCycle::drawCycle()
{	
	glPushMatrix();
	
		glTranslatef( position[0], position[1], position[2] );
		glRotatef( angle, 0.0, 0.0, 1.0 );

		// Wheels
		// front base
		glColor3f( color[0], color[1], color[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -bodysize + 0.05, 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -bodysize + 0.05, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -bodysize - wheelsize + 0.05, 0.0 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -bodysize - wheelsize + 0.05, 0.0 );
		glEnd();
		
		// front rims
		glColor3f( lightcolor[0], lightcolor[1], lightcolor[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.02, -bodysize + 0.05, 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.02, -bodysize + 0.05, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.02, -bodysize - wheelsize + 0.05, 0.0 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.02, -bodysize - wheelsize + 0.05, 0.0 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -bodysize + 0.02, 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -bodysize + 0.02, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -bodysize - wheelsize + 0.08, 0.0 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -bodysize - wheelsize + 0.08, 0.0 );
		glEnd();

		
		// back base interior
		glColor3f( color[0], color[1], color[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 2.5) - (bodysize / 6.0), -tailsize - wheelsize, 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 2.5), -tailsize - wheelsize, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 2.5), -tailsize - wheelsize + 0.05, 0.0 );	
			glVertex3f( -(wheelsize / 2.5) - (bodysize / 6.0), -tailsize - wheelsize + 0.05, 0.0 );
		glEnd();
		// back base exterior
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - wheelsize, 0.0 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - wheelsize, 0.0 );
		glEnd();
		// back base tail
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize , 0.0 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize, 0.0 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize, 0.0 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize, 0.0 );
		glEnd();
	
	
		// Main Body
		glColor3f( color[0], color[1], color[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0), -bodysize - tailsize, 0.01 );
			glVertex3f( (bodysize / 6.0), -bodysize - tailsize, 0.01 );	
			glVertex3f( (bodysize / 6.0), -tailsize, 0.01 );	
			glVertex3f( -(bodysize / 6.0), -tailsize, 0.01 );
		glEnd();
		
		// black body
		// back square
		glColor3f( 0.0, 0.0, 0.0 );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.02, -tailsize - wheelsize - 0.025, 0.02 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.02, -tailsize - wheelsize - 0.025, 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.02, -tailsize - 0.02, 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.02, -tailsize - 0.02, 0.02 );
		glEnd();
		// black body - back rectangle
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize - 0.025, 0.02 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize - 0.025, 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize + 0.02, 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize + 0.02, 0.02 );
		glEnd();
		// black body - long middle
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.04, -tailsize - bodysize + 0.025, 0.02 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.04, -tailsize - bodysize + 0.025, 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.04, -tailsize - wheelsize - 0.025, 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.04, -tailsize - wheelsize - 0.025, 0.02 );
		glEnd();
		// black body - front rectangle
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize, 0.02 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize, 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.025, 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.025, 0.02 );
		glEnd();
		
		
		// black body - left arm
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - ((bodysize / 2.0) * 1.35), 0.02 );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.03, -tailsize - ((bodysize / 2.0) * 1.35), 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.03, -tailsize - wheelsize, 0.02 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize, 0.02 );
		glEnd();
		// black body - right arm
		glBegin( GL_POLYGON );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.03, -tailsize - ((bodysize / 2.0) * 1.35), 0.02 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - ((bodysize / 2.0) * 1.35), 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize, 0.02 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.03, -tailsize - wheelsize, 0.02 );
		glEnd();
		
		
		
		// grill
		glColor3f( color[0], color[1], color[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0 * 0.75), -bodysize - tailsize - 0.03, 0.01 );
			glVertex3f( (bodysize / 6.0 * 0.75), -bodysize - tailsize - 0.03, 0.01 );	
			glVertex3f( (bodysize / 6.0 * 0.75), -bodysize - tailsize, 0.01 );	
			glVertex3f( -(bodysize / 6.0 * 0.75), -bodysize - tailsize, 0.01 );
		glEnd();
		
		// driver - front end
		glColor3f( lightcolor[0], lightcolor[1], lightcolor[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.025, 0.03 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.025, 0.03 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.04, 0.03 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.04, 0.03 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( 0.01, -tailsize - bodysize + 0.04, 0.03 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.04, 0.03 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.06, 0.03 );	
			glVertex3f( 0.02, -tailsize - bodysize + 0.055, 0.03 );
		glEnd();
		glColor3f( shading[0], shading[1], shading[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.04, 0.03 );
			glVertex3f( 0.02, -tailsize - bodysize + 0.04, 0.03 );	
			glVertex3f( 0.01, -tailsize - bodysize + 0.06, 0.03 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.055, 0.03 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.055, 0.03 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.055, 0.03 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.05, -tailsize - bodysize + 0.07, 0.03 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.05, -tailsize - bodysize + 0.07, 0.03 );
		glEnd();
		
		// driver - back end
		glColor3f( lightcolor[0], lightcolor[1], lightcolor[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0 * 0.5), -tailsize - wheelsize - 0.01, 0.04 );
			glVertex3f( (bodysize / 6.0 * 0.5), -tailsize - wheelsize - 0.01, 0.04 );	
			glVertex3f( (bodysize / 6.0 * 0.5), -tailsize - 0.06, 0.04 );	
			glVertex3f( -(bodysize / 6.0 * 0.5), -tailsize - 0.06, 0.04 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0 * 0.5) - 0.015, -tailsize - wheelsize + 0.02, 0.04 );
			glVertex3f( (bodysize / 6.0 * 0.5) + 0.015, -tailsize - wheelsize + 0.02, 0.04 );	
			glVertex3f( (bodysize / 6.0 * 0.5) + 0.015, -tailsize - 0.06, 0.04 );	
			glVertex3f( -(bodysize / 6.0 * 0.5) - 0.015, -tailsize - 0.06, 0.04 );
		glEnd();
		
		// tail
		glColor3f( shading[0], shading[1], shading[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0 * 0.5), -tailsize, 0.03 );
			glVertex3f( (bodysize / 6.0 * 0.5), -tailsize, 0.03 );	
			glVertex3f( (bodysize / 6.0 * 0.5), 0.0, 0.03 );	
			glVertex3f( -(bodysize / 6.0 * 0.5), 0.0, 0.03 );
		glEnd();
		glColor3f( color[0], color[1], color[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(bodysize / 6.0 * 0.5), -tailsize - 0.04, 0.04 );
			glVertex3f( (bodysize / 6.0 * 0.5), -tailsize - 0.04, 0.04 );	
			glVertex3f( (bodysize / 6.0 * 0.5), -tailsize + 0.02, 0.04 );	
			glVertex3f( -(bodysize / 6.0 * 0.5), -tailsize + 0.02, 0.04 );
		glEnd();
		
		//details
		// left arm detail
		glColor3f( lightcolor[0], lightcolor[1], lightcolor[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.015, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.015, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - (bodysize / 2.0) - 0.03, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - (bodysize / 2.0) - 0.01, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) - 0.01, 0.04 );
		glEnd();
		
		// right arm detail
		glBegin( GL_POLYGON );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.015, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.015, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );
		glEnd();
		glBegin( GL_POLYGON );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) - 0.03, 0.04 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - (bodysize / 2.0) - 0.03, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - (bodysize / 2.0) - 0.01, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) - 0.01, 0.04 );
		glEnd();
		
		// left arm detail
		glColor3f( shading[0], shading[1], shading[2] );
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.015, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.015, -tailsize - (bodysize / 2.0) + 0.03, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) - 0.04, -tailsize - (bodysize / 2.0) + 0.03, 0.04 );
		glEnd();
		
		// right arm detail
		glBegin( GL_POLYGON );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.015, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) + 0.01, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.04, -tailsize - (bodysize / 2.0) + 0.03, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) + 0.015, -tailsize - (bodysize / 2.0) + 0.03, 0.04 );
		glEnd();
		
		// tail details
		// left
		glBegin( GL_POLYGON );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize + 0.02 , 0.04 );
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.025, -tailsize - wheelsize + 0.02, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0) + 0.025, -tailsize - wheelsize + 0.06, 0.04 );	
			glVertex3f( -(wheelsize / 4.5) - (bodysize / 6.0), -tailsize - wheelsize + 0.06, 0.04 );
		glEnd();
		// right
		glBegin( GL_POLYGON );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.025, -tailsize - wheelsize + 0.02 , 0.04 );
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize + 0.02, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5), -tailsize - wheelsize + 0.06, 0.04 );	
			glVertex3f( (bodysize / 6.0) + (wheelsize / 4.5) - 0.025, -tailsize - wheelsize + 0.06, 0.04 );
		glEnd();

	glPopMatrix();

}

// Compute tank position
void LightCycle::computePosition( double elapsed_time )
{
	if ( disableGame )
	{
		return;
	}
	
	float velocityX;
	float velocityY;

	if ( elapsed_time > MINIMUM_MILLISECOND_UPDATE )
	{
		// set direction vehicle is allowed to move on certain angles
		
		if (angle == 0.0 || angle == 360.0)
		{
			moveHorizon = 0.0;
			moveVertical = -1.0;
		}
		if (angle == 90.0)
		{
			moveHorizon = 1.0;
			moveVertical = 0.0;
		}
		if (angle == 180.0)
		{
			moveHorizon = 0.0;
			moveVertical = 1.0;
		}
		if (angle == 270.0)
		{
			moveHorizon = -1.0;
			moveVertical = 0.0;
		}
		
		velocityX = moveHorizon * speed * elapsedTime;
		velocityY = moveVertical * speed * elapsedTime;
		
		prevPosition[0] = position[0];
		prevPosition[1] = position[1];
		prevPosition[2] = position[2];

		position[0] += velocityX;
		position[1] += velocityY;
		
		// calculate offset for disc creation
		posOffsetX = 0.0;
		posOffsetY = bodysize + tailsize + 0.1;
		
		// create light trail behind light cycle
		int numTrail = gTrails.size();

		if ( identify == 0)
		{
			gTrails.push_back(  new Trail( position[0], position[1], position[2], lightcolor[0], lightcolor[1], lightcolor[2] ) );
		}
		if ( identify == 1)
		{
			gTrails.push_back(  new Trail( position[0], position[1], position[2], color[0], color[1], color[2] ) );
		}
		
		gTrails[numTrail]->position[0] = position[0];
		gTrails[numTrail]->position[1] = position[1];		
		gTrails[numTrail]->position[2] = 0;
		gTrails[numTrail]->angle = angle;
			
	}

}

// Compute intersection with the other tank
// If intersection occurs, set back to previous position and turn direction away from possible collision
void LightCycle::computeIntersection()
{
	if ( disableGame )
	{
		return;
	}
	
	for ( unsigned int i = 0; i < gCycles.size(); ++i )
	{
		
		// if the bounding boxes of each light cycle are overlapping on all axis, they are intersecting
		if ( (gCycles[1]->position[0] + sizeW) >= (gCycles[0]->position[0] - sizeW) && (gCycles[1]->position[0] - sizeW) <= (gCycles[0]->position[0] + sizeW) && (gCycles[1]->position[1] + sizeL) >= (gCycles[0]->position[1]) && (gCycles[1]->position[1]) <= (gCycles[0]->position[1] + sizeL))
		{
			gCycles[i]->position[0] = gCycles[i]->prevPosition[0];
			gCycles[0]->angle += 90;
			gCycles[i]->position[1] = gCycles[i]->prevPosition[1];
			gCycles[1]->angle += 90;
		}	
	}
}

// fire identity disk
void LightCycle::fireDisc()
{
	if ( disableGame )
	{
		return;
	}
	
	if ( disabled )
	{
		return;
	}

	clock_t current_time = clock();
	double betweenfire_time = ( double )( diffclock( current_time, lastTimeFired ) ) / 1000.0;	

	// if the time between shots is greater than the set delay, shoot the next disc
	if ( betweenfire_time > shotDelay )
	{
		lastTimeFired = current_time;
		
			// add a disc to the vector of disc
			// set positon, angle
			int numDisc = gDiscs.size();
			
			// set color based on which vehicle the light trail belongs to
			if ( identify == 0)
			{
				gDiscs.push_back(  new Disc( position[0], position[1], position[2], lightcolor[0], lightcolor[1], lightcolor[2] ) );
			}
			if ( identify == 1)
			{
				gDiscs.push_back(  new Disc( position[0], position[1], position[2], color[0], color[1], color[2] ) );
			}
			
			gDiscs[numDisc]->angle = angle;
			// decide offset of disc fire based on angle the light cycle is facing
			if (angle == 90)
			{
				gDiscs[numDisc]->position[0] = position[0] + sizeL + 0.2 + (posOffsetX * sin(angle * M_PI / 180));	
				gDiscs[numDisc]->position[1] = position[1] + (posOffsetY * cos(angle * M_PI / 180));
				gDiscs[numDisc]->position[2] = 0;
			}
			if (angle == 180)
			{
				gDiscs[numDisc]->position[0] = position[0] + (posOffsetX * sin(angle * M_PI / 180));	
				gDiscs[numDisc]->position[1] = position[1] + (sizeL * 2) + 0.2 + (posOffsetY * cos(angle * M_PI / 180));
				gDiscs[numDisc]->position[2] = 0;
			}
			if (angle == 270)
			{
				gDiscs[numDisc]->position[0] = position[0] - sizeL - 0.2 + (posOffsetX * sin(angle * M_PI / 180));	
				gDiscs[numDisc]->position[1] = position[1] + (posOffsetY * cos(angle * M_PI / 180));
				gDiscs[numDisc]->position[2] = 0;
			}
			if (angle == 0 || angle == 360)
			{
				gDiscs[numDisc]->position[0] = position[0] + (posOffsetX * sin(angle * M_PI / 180));	
				gDiscs[numDisc]->position[1] = position[1] - (sizeL * 2) - 0.2 + (posOffsetY * cos(angle * M_PI / 180));
				gDiscs[numDisc]->position[2] = 0;
			}
	}
}
	
// do not fire disc
void LightCycle::disableDisc()
{
	disabledTime = clock();
}

// keyDown
// 
// The tank currently will set moving to be 1 when the move forward key (s for the tank set up now) is pressed 
// This value is then used in computePosition

void LightCycle::keyDown( char key )
{
	if ( key == leftKey ) 
	{
			// if the user hits the left key turn back 90 degrees
			angle -= 90.0;
			// if the angle goes below zero than add 360
			if (angle < 0)
			{
				angle += 360;
			}
	}
	if ( key == rightKey ) 
	{
			// if the user hits the right key turn forward 90 degrees
			angle += 90.0;
			// if the angle goes above 360 than subtract 360 to reset
			if (angle > 360)
			{
				angle -= 360;
			}
	}
	if ( key == discKey )
	{
			fireDisc();
	}

}

// keyUp - do nothing
void LightCycle::keyUp( char key )
{
	if ( key == leftKey ) 
	{
			
	}
	if ( key == rightKey ) 
	{

	}
	if ( key == discKey )
	{
			disableDisc();
	}

}

// Check if any of the light cycles have run out of lives
// If they have end the game and announce the winner
void LightCycle::endGame()
{
	if ( disableGame )
	{
		return;
	}
	
	for ( unsigned int i = 0; i < gCycles.size(); ++i )
	{
		if (gCycles[i]->lives <= 0)
		{
			if (gCycles[0]->lives == 0)
			{
				cout << "\nUser Two Wins!!" << endl;
			}
			if (gCycles[1]->lives == 0)
			{
				cout << "\nUser One Wins!!" << endl;	
			}
			
			disableGame = true;
		}
	}
}

