// Include appropriate libraries
#include <iostream>
#include <GL/glut.h>
#include <vector>
#include <math.h>

// Include other classes required
#include "LightCycle.h"
#include "Trail.h"
#include "Disc.h"
#include "graphicsUtils.h"

using namespace std;

// Grab access to the global list of pointers to objects
extern vector< LightCycle* > gCycles;
extern vector< Trail* > gTrails;
extern vector< Disc* > gDiscs;

// Trail CLASS 

Trail::Trail( float x = 0.0, float y = 0.0, float z = 0.0, float color0 = 0.0, float color1 = 0.0, float color2 = 0.0 )
{
	position[0] = x;
	position[1] = y;
	position[2] = z;
	
	color[0] = color0;
	color[1] = color1;
	color[2] = color2;
	
	angle = 0;
}

// Draw light trail
void Trail::drawTrail()
{
	glPushMatrix();
	
	
	glTranslatef( position[0], position[1], position[2] );
	glRotatef( angle, 0.0, 0.0, 1.0 );
	
		glBegin( GL_QUADS );
			glColor3f( color[0], color[1], color[2] );
			glVertex3f( -(0.4 / 6.0 * 0.5), 0.02, 0.05 );	
			glVertex3f( (0.4 / 6.0 * 0.5), 0.02, 0.05 );	
			glVertex3f( (0.4 / 6.0 * 0.5), 0.07, 0.05 );	
			glVertex3f( -(0.4 / 6.0 * 0.5), 0.07, 0.05 );	
		glEnd();
		
	glPopMatrix();

}

// Compute intersection of light cycles and discs with light trails
void Trail::computeIntersection(int On, int iPos)
{	
	//float xmin, xmax, ymin, ymax;
	
	//// Light Cycles
	//for( unsigned int i = 0; i < gCycles.size(); i++)
	//{			
		//xmin = position[0] - (0.4 / 6.0 * 0.5);
		//xmax = position[0] + (0.4 / 6.0 * 0.5);
		//ymin = position[1];
		//ymax = position[1] + 0.05;
		
		//for( unsigned int j = 0; j < gTrails.size(); j++)
		//{
			//if ( gCycles[i]->angle == 0 || gCycles[i]->angle == 180 || gCycles[i]->angle == 360 )
			//{
				//// If light cycle hits barrier, delete it and end game
				//if (gCycles[i]->position[0] > xmin && gCycles[i]->position[0] < xmax)
				//{
					//if ( gCycles[i]->position[1] + gCycles[i]->sizeL >= ymin || gCycles[i]->position[1] - gCycles[i]->sizeL <= ymax)
					//{
						//gCycles[i]->position[0] = gCycles[i]->prevPosition[0];
						//gCycles[i]->position[1] = gCycles[i]->prevPosition[1];
						//cout << "destroy" << endl;
					//}
				//}
			//}
			//if ( gCycles[i]->angle == 90 || gCycles[i]->angle == 270 )
			//{
				//// If light cycle hits barrier, delete it and end game
				//if (gCycles[i]->position[1] > ymin && gCycles[i]->position[1] < ymax)
				//{
					//if ( gCycles[i]->position[0] + gCycles[i]->sizeL >= xmin || gCycles[i]->position[0] - gCycles[i]->sizeL <= xmax)
					//{
						//gCycles[i]->position[0] = gCycles[i]->prevPosition[0];
						//gCycles[i]->position[1] = gCycles[i]->prevPosition[1];
					//}
				//}
			//}
		//}
	//}
	
	if (On == 0)
	{
		// Discs
		float templeft, tempright, temptop, tempbottom;
		
		for( unsigned int k = 0; k < gDiscs.size(); k++)
		{
			templeft = (position[0] - (0.4 / 6.0 * 0.5)) - gDiscs[k]->size;
			tempright = (position[0] + (0.4 / 6.0 * 0.5)) + gDiscs[k]->size;
			temptop = (position[1] + 0.05) + gDiscs[k]->size;
			tempbottom = (position[1]) - gDiscs[k]->size;
			
			// If disc hits light trail, delete it
			if (gDiscs[k]->position[0] > templeft && gDiscs[k]->position[0] < tempright && gDiscs[k]->position[1] < temptop && gDiscs[k]->position[1] > tempbottom)
			{
				
				if (gDiscs[k]->prevPosition[0] < templeft || gDiscs[k]->prevPosition[0] > tempright)
				{
				}
				if (gDiscs[k]->prevPosition[1] > temptop || gDiscs[k]->prevPosition[1] < tempbottom)
				{
				}

				gTrails.erase(gTrails.begin() + iPos);
				delete(this);
			}
		}
	}
}
