// Graphics Utilities

#include <GL/glut.h>
#include "graphicsUtils.h"
#include <ctime>
#include <vector>

// FUNCTION FOR TRACKING TIME

double diffclock( clock_t clock1, clock_t clock2 )
{
        double diffticks = clock1 - clock2;
        double diffms = ( diffticks * 1000 ) / CLOCKS_PER_SEC;
        return diffms;
}


// CLASS FOR SETTING UP CAMERA - MEMBER FUNCTIONS

Camera::Camera()
{
        aspect = 1;
        windowWidth = 1024;
        windowHeight = 1024;
        orthoHeight = 20.0;

        fov = 45.0;
        aspect = 1.0;
        nearval = 1.5;
        farval = 50.0;
}

void Camera::doCamera()
{
        glLoadIdentity();
        // gluLookAt location, look at, up vector values
        gluLookAt(  0.0, 0.0, 5.0  ,  0.0, 0.0, 0.0  ,  0.0, 1.0, 0.0  );

}

void Camera::doCameraPerspective( int w, int h )
{
        aspect = ( float ) windowWidth / ( float ) windowHeight;
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        gluPerspective( fov, aspect, nearval, farval );
}

void Camera::doCameraOrthographic( int w, int h)
{
        windowWidth = w;
        windowHeight = h;
        aspect = ( float ) w / ( float ) h ;
        float top = orthoHeight / 2.0;
        float bottom = 0.0;
        float right = ( orthoHeight * aspect ) /2.0;
        float left = 0.0;

        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        glOrtho( left, right, bottom, top, nearval, farval );
} 

Grid::Grid()
{
	// set default coordinates at the bottom left of the screen
	squarex = 0.0;
	squarey = 0.0;
	squarez = 0.0;
	
	gridMax = 20;
	
	scoreDisplay = (5.0 / 20) / gridMax;
	
	// initialize space between squares
	space = 0.01;
	// initialize box size based on the display size divided by the number of squares
	size = (10.0 / gridMax) - scoreDisplay;
	
	// set up an edge border so game is not on the edge of the screen
	edge = (size / 2.0) + (scoreDisplay * gridMax / 2.0);
}

// draw one square
void Grid::drawSquare( float squarex, float squarey, float squarez, float size, float space )
{
	// draw sqaure based on x and y values, and size brought in from draw grid function
	glBegin( GL_POLYGON );
		glVertex3f( squarex , squarey, squarez );
		glVertex3f( squarex + size - space, squarey, squarez );	
		glVertex3f( squarex + size - space, squarey + size - space, squarez );	
		glVertex3f( squarex, squarey + size - space, squarez );
	glEnd();
}

void Grid::drawGrid()
{		
	glPushMatrix();
	
	// draw the grid
	for (int columnNum = 0; columnNum < gridMax; ++columnNum)
	{
		// calculate position of x value
		squarex = columnNum * size + edge;
		
		// loop to draw rows
		for (int rowNum = 0; rowNum < gridMax; ++rowNum)
		{
			// calculate position of y value
			squarey = rowNum * size + (scoreDisplay * gridMax / 4.0);

		glColor3f( color[0][0], color[0][1], color[0][2] );	
		drawSquare( squarex, squarey, squarez + 0.02, size, space );
		glColor3f( 0.0, 0.0, 0.0 );	
		drawSquare( squarex + 0.01, squarey + 0.01, squarez + 0.03, size - 0.02, space );
		
		}
	}
	glPopMatrix();

}

// table holding color values for easy access
void Grid::colortable()
{
	// light turquoise - neon user 1
	color[0][0] = 0.819607;
	color[0][1] = 1.0;
	color[0][2] = 0.964705;
	
	// turquoise
	color[1][0] = 0.670588;
	color[1][1] = 0.909803;
	color[1][2] = 0.890196;
	
	// light yellow - neon user 2
	color[2][0] = 1.0;
	color[2][1] = 0.878431;
	color[2][2] = 0.250980;
	
	// yellow orange
	color[3][0] = 1.0;
	color[3][1] = 0.501960;
	color[3][2] = 0.098039;
	
	// white
	color[4][0] = 1.0;
	color[4][1] = 1.0;
	color[4][2] = 1.0;

}

